import json
from django.http import HttpResponse
from grid_registrar import get_grid_class

__author__ = 'zmbq'

class JsonResponse(HttpResponse):
    def __init__(self, content, mimetype='application/json', status=None, content_type='application/json'):
        super(JsonResponse, self).__init__(
            content=json.dumps(content),
            mimetype=mimetype,
            status=status,
            content_type=content_type)


def query(request, grid_id):
    cls = get_grid_class(grid_id)
    grid = cls()
    data = grid.get_json_data(request.GET)
    return JsonResponse(data)



import string
from django.template import Context, Template
from django.template.loader import get_template
from json_helpers import function

__author__ = 'zmbq'

class Column(object):
    _creation_count = 0

    def __init__(self, title, model_path, **kwargs):
        self._title = title
        self._colModel = dict(kwargs)
        self._colModel['name'] = title.lower()
        if not 'formatter' in kwargs:
            self._colModel['formatter'] = function('customHtmlFormatter')
        self._count = Column._creation_count
        Column._creation_count += 1
        self._model_path = model_path

    def get_model_attr(self, attr, model):
        return reduce(getattr, attr.split('.'), model)

    def get_model_value(self, model):
        return self.get_model_attr(self._model_path, model)

    def render_text(self, model):
        return str(self.get_model_value(model))

    def render_html(self, model):
        return self.render_text(model)

    @property
    def title(self):
        """ Returns the name that goes in the colName JSON """
        return self._title

    @property
    def model(self):
        return self._colModel

    def get_sort_name(self):
        return self._model_path.replace('.', '__')

class TextColumn(Column):
    def __init__(self, title, model_path, **kwargs):
        super(TextColumn, self).__init__(title, model_path, **kwargs)
        self._model_path = model_path
        del self._colModel['formatter'] # Use the default formatter

class ClientOnlyColumn(Column):
    """ A column that is filled by the client, the server just returns it with a blank value """
    def __init__(self, title, **kwargs):
        super(ClientOnlyColumn, self).__init__(title, None, **kwargs)
        if not 'formatter' in kwargs:
            del self._colModel['formatter']

    def get_model_value(self, model):
        return ''

class TemplateColumn(Column):
    def __init__(self, title, model_path, template = None, template_name = None,**kwargs):
        super(TemplateColumn, self).__init__(title, model_path, **kwargs)
        if template:
            self._template = Template(template.strip())
        elif template_name:
            self._template = get_template(template_name)
        else:
            self._template = None # set_template may be called later

    def set_template(self, template):
        self._template = Template(template.strip())

    def fill_context(self, context, model):
        return context

    def render_html(self, model):
        if not self._template:
            raise ValueError("Template has not been set in %s" % self)
        context = Context({'model': model})
        self.fill_context(context, model)
        html = self._template.render(context = context)

        lines = html.split('\n')
        lines = [line.strip() for line in lines]
        html = string.join(lines)

        return html

class LinkColumn(TemplateColumn):
    template = """
    <a href="{{ url }}">{{ name }}</a>
    """

    def __init__(self, title, model_path, url_builder, **kwargs):
        super(LinkColumn, self).__init__(title, model_path, template=LinkColumn.template, **kwargs)
        self._url_builder = url_builder

    def fill_context(self, context, model):
        url = self._url_builder(model)
        context['url'] = url
        context['name'] = self.get_model_attr(self._model_path, model)

class CheckboxColumn(Column):
    def __init__(self, title, model_path, **kwargs):
        kwargs['formatter'] = 'checkbox'
        if 'editable' in kwargs and not 'edittype' in kwargs:
            kwargs['formatoptions'] = {'disabled': False}
            kwargs['edittype'] = 'checkbox'
            kwargs['editoptions'] = {'value': 'true:false'}
        super(CheckboxColumn, self).__init__(title, model_path, **kwargs)

    def render_text(self, model):
        text = super(CheckboxColumn, self).render_text(model)
        return text=='True'    # Turn the field into a boolean, for JavaScript

class KeyColumn(Column):
    def __init__(self, model_path, **kwargs):
        kwargs['key'] = True
        kwargs['hidden'] = True
        super(KeyColumn, self).__init__("Key", model_path, **kwargs)



# coding=utf-8
import json
from django import template
from djqgrid import json_helpers

register = template.Library()

@register.simple_tag(takes_context=True)
def jqgrid(context, grid, prefix='', pager=True, urlquery=None, **kwargs):
    if prefix and prefix[-1]!='-':
        prefix += '-'  # Append a - to the prefix

    gridId = prefix + "grid"
    pagerId = prefix + "pager"
    options = grid.get_options(kwargs)
    # Add the query dict to the url
    if not '?' in options['url']:
        options['url'] += '?'
    if urlquery:
        options['url'] += urlquery
    options['url'] += context['request'].GET.urlencode()
    if pager:
        options['pager'] = '#' + pagerId
    else:
        options['rows'] = 99999
    options = json_helpers.dumps(options, indent=4)

    html = """
    <table id="%s"><tr><td></td></tr></table>
    <div id="%s"></div>
    <script type="text/javascript">
        $(function() {
            $('#%s').jqGrid(%s);
        });
    </script>""" % (gridId, pagerId, gridId, options);

    return html



import copy
from zlib import crc32
from django.core.paginator import Paginator
from django.core.urlresolvers import reverse
from django.utils import six
from django.utils.datastructures import SortedDict
import views
from columns import Column
from grid_registrar import register_grid

__author__ = 'zmbq'

# The meta class implementation is taken from the Django Form class, only with name changes
def get_declared_columns(bases, attrs, with_base_columns=True):
    """
    Create a list of form column instances from the passed in 'attrs', plus any
    similar columns on the base classes (in 'bases'). This is used by Grid metaclass.

    If 'with_base_columns' is True, all columns from the bases are used.
    Otherwise, only columns in the 'declared_columns' attribute on the bases are
    used. The distinction is useful in ModelForm subclassing.
    Also integrates any additional media definitions.
    """
    columns = [(column_name, attrs.pop(column_name)) for column_name, obj in list(six.iteritems(attrs)) if isinstance(obj, Column)]
    columns.sort(key=lambda x: x[1]._count)

    # If this class is subclassing another Form, add that Form's columns.
    # Note that we loop over the bases in *reverse*. This is necessary in
    # order to preserve the correct order of columns.
    if with_base_columns:
        for base in bases[::-1]:
            if hasattr(base, 'base_columns'):
                columns = list(six.iteritems(base.base_columns)) + columns
    else:
        for base in bases[::-1]:
            if hasattr(base, 'declared_columns'):
                columns = list(six.iteritems(base.declared_columns)) + columns

    return SortedDict(columns)

class DeclarativeColumnsMetaclass(type):
    """
    Metaclass that converts Column attributes to a dictionary called
    'base_fields', taking into account parent class 'base_fields' as well.
    """
    def __new__(cls, name, bases, attrs):
        attrs['base_columns'] = get_declared_columns(bases, attrs)
        new_class = super(DeclarativeColumnsMetaclass,
                     cls).__new__(cls, name, bases, attrs)
        return new_class

class BaseGrid(object):
    _default_options = dict(datatype = 'json',
                            mtype = 'get',
                            viewrecords = True,
                            gridview = True,
                            autoencode= False,
                            height = '100%',
                            autowidth = True,
                            shrinkToFit = True,
                            rowNum = 20)

    def __init__(self, **kwargs):
        self._columns = copy.deepcopy(self.base_columns)
        self._options = BaseGrid._default_options.copy()
        self._options.update(kwargs)
        register_grid(self.__class__)

    @classmethod
    def get_grid_id(cls):
        return '%X' % abs(crc32(cls.__name__))

    @property
    def columns(self):
        return self._columns

    def get_options(self, override = None):
        options = self._options.copy()
        if override:
            options.update(override)

        options['colNames'] = [col.title for col in self.columns.values()]
        options['colModel'] = [col.model for col in self.columns.values()]
        options['url'] = reverse(views.query, kwargs = {'grid_id': self.get_grid_id()})

        return options

    def model_to_dict(self, model):
        result = {}
        html = {}
        for column in self.columns.values():
            title = column.title.lower()
            result[title] = column.render_text(model)
            html[title] = column.render_html(model)
        result['html'] = html

        additional = self.get_additional_data(model)
        if additional:
            result['additional'] = additional
        return result

    def get_additional_data(self, model):
        return None

    def apply_sort(self, queryset, querydict):
        try:
            sidx = querydict['sidx']
            if not sidx:
                sidx = self.columns.values()[0].title
            sorder = '' if querydict['sord']=='asc' else '-'
        except KeyError:
            return queryset  # No sorting applied
        for column in self.columns.values():
            if sidx.lower()==column.title.lower():
                order = sorder + column.get_sort_name()
                return queryset.order_by(order)
        raise ValueError("Can't find index field '%s'" % sidx)

    def apply_query(self, queryset, querydict):
        return queryset

    def get_query_results(self, querydict):
        queryset = self.model.objects
        queryset = self.apply_query(queryset, querydict)
        queryset = self.apply_sort(queryset, querydict)

        return queryset

    def get_json_data(self, querydict):
        queryset = self.get_query_results(querydict)

        page = int(querydict.get('page', '1'))
        rows = int(querydict.get('rows', self._options.get('rownum', 20)))
        queryset = self.apply_sort(queryset, querydict)

        total_pages = (len(queryset) + rows - 1) / rows
        paginator = Paginator(queryset, rows)
        response = {'page': page,
                    'total': total_pages,
                    'records': len(queryset),
                    'rows': [self.model_to_dict(m) for m in paginator.page(page)]}
        return response
    

class Grid(six.with_metaclass(DeclarativeColumnsMetaclass, BaseGrid)):
    """ Adds the fields metaclass magic, as is done in the Django forms """







import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
# LICENSE = open(os.path.join(os.path.dirname(__file__), 'LICENSE.txt')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='djqgrid',
    version='0.1',
    packages=['djqgrid', 'djqgrid.templatetags'],
    include_package_data=True,
    license='MIT license',
    description='A Django wrapper for jqGrid',
    long_description=README,
    url='https://github.com/zmbq/djqgrid/',
    author='Itay Zandbank',
    author_email='zmbq@platonix.com',
    install_requires=['django>=1.6'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Development Status :: 3 - Alpha',
    ],
    keywords='django jqgrid client-side grid',
)


import json

__author__ = 'zmbq'

escape = '@@'

def function(funcname):
    return escape + funcname + escape

def dumps(o, *args, **kwargs):
    s = json.dumps(o, *args, **kwargs)
    s = s.replace('"' + escape, '')
    s = s.replace(escape + '"', '')
    return s